package com.om.controller.admin;


import com.om.entity.dto.DeviceQueryPageDTO;
import com.om.entity.po.Device;
import com.om.entity.vo.DeviceQueryPageVO;
import com.om.service.IDeviceService;
import com.om.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author bmmx
 * @since 2024-04-08
 */
@RestController
@Api(tags = "平板设备管理")
@RequestMapping("/web/device")
public class DeviceController {

    @Resource
    private IDeviceService deviceService;

    @ApiOperation("分页查询接口")
    @PostMapping("getPageList")
    public Result<DeviceQueryPageVO> getPageList(@RequestBody DeviceQueryPageDTO dto) {
        return deviceService.getPageList(dto);
    }

    @ApiOperation("查询集合接口")
    @GetMapping("list")
    public Result<List<String>> list() {
        return Result.ok(deviceService.deviceList());
    }

    @ApiOperation("根据id查询平板设备")
    @GetMapping("getById")
    public Result<Device> getById(@RequestParam("id") Integer id) {
        return Result.ok(deviceService.getById(id));
    }

    @ApiOperation("新增平板设备")
    @PostMapping("add")
    public Result add(@RequestBody Device device) {
        device.setCreateTime(LocalDateTime.now());
        return Result.ok(deviceService.save(device));
    }

    @ApiOperation("根据id修改平板设备")
    @PostMapping("update")
    public Result update(@RequestBody Device device) {
        return Result.ok(deviceService.updateById(device));
    }

    @ApiOperation("根据id删除平板设备")
    @GetMapping("delete")
    public Result delete(@RequestParam("id") Integer id) {
        return Result.ok(deviceService.removeById(id));
    }
}
